==> server-24.1.0-rc.1-sources.jar: com/xebialabs/deployit/server/jetty/metrics/CustomInstrumentedHandler.java <==
package com.xebialabs.deployit.server.jetty.metrics;

import com.tqdev.metrics.core.Gauge;
import com.tqdev.metrics.core.MetricRegistry;
import jakarta.servlet.AsyncEvent;
import jakarta.servlet.AsyncListener;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.eclipse.jetty.http.HttpMethod;
import org.eclipse.jetty.server.AsyncContextState;
import org.eclipse.jetty.server.HttpChannelState;
import org.eclipse.jetty.server.Request;
import org.eclipse.jetty.server.handler.HandlerWrapper;
import org.eclipse.jetty.util.thread.QueuedThreadPool;

import java.io.IOException;

public class CustomInstrumentedHandler extends HandlerWrapper {

    /** The registry to store metrics in. */
    private final MetricRegistry registry;

    /**
     * Instantiates a new instrumented handler.
     *
     * @param registry
     *            the registry
     */
    public CustomInstrumentedHandler(MetricRegistry registry) {
        this.registry = registry;
    }

    /** The listener to handle async requests consistently. */
    private AsyncListener listener = new AsyncListener() {
        private long startTime;

        @Override
        public void onTimeout(AsyncEvent event) throws IOException {
        }

        @Override
        public void onStartAsync(AsyncEvent event) throws IOException {
            startTime = registry.getNanos();
            event.getAsyncContext().addListener(this);
        }

        @Override
        public void onError(AsyncEvent event) throws IOException {
        }

        @Override
        public void onComplete(AsyncEvent event) throws IOException {
            final AsyncContextState state = (AsyncContextState) event.getAsyncContext();
            final HttpServletRequest request = (HttpServletRequest) state.getRequest();
            final HttpServletResponse response = (HttpServletResponse) state.getResponse();
            updateResponses(request, response, startTime);
        }
    };

    /*
     * (non-Javadoc)
     *
     * @see org.eclipse.jetty.server.handler.AbstractHandler#doStart()
     */
    @Override
    protected void doStart() throws Exception {
        super.doStart();

        for (int responseStatus = 1; responseStatus <= 5; responseStatus++) {
            registry.set("jetty.Response.Invocations", responseStatus + "xx-responses", 0);
            registry.set("jetty.Response.Durations", responseStatus + "xx-responses", 0);
        }
        registry.set("jetty.Response.Invocations", "other-responses", 0);
        registry.set("jetty.Response.Durations", "other-responses", 0);
        for (HttpMethod method : HttpMethod.values()) {
            String name = method.asString().toLowerCase();
            registry.set("jetty.Request.Invocations", name + "-requests", 0);
            registry.set("jetty.Request.Durations", name + "-requests", 0);
        }
        registry.set("jetty.Request.Invocations", "other-requests", 0);
        registry.set("jetty.Request.Durations", "other-requests", 0);
        registry.set("jetty.Aggregated.Invocations", "requests", 0);
        registry.set("jetty.Aggregated.Durations", "requests", 0);

        registry.set("jetty.Thread.Gauges", "threads", (Gauge) () -> getServer().getThreadPool().getThreads());
        registry.set("jetty.Thread.Gauges", "idle-threads", (Gauge) () -> getServer().getThreadPool().getIdleThreads());
        if (getServer().getThreadPool() instanceof QueuedThreadPool) {
            registry.set("jetty.Thread.Gauges", "busy-threads",
                    (Gauge) () -> ((QueuedThreadPool) getServer().getThreadPool()).getBusyThreads());
            registry.set("jetty.Thread.Gauges", "min-threads",
                    (Gauge) () -> ((QueuedThreadPool) getServer().getThreadPool()).getMinThreads());
            registry.set("jetty.Thread.Gauges", "max-threads",
                    (Gauge) () -> ((QueuedThreadPool) getServer().getThreadPool()).getMaxThreads());
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.eclipse.jetty.server.handler.HandlerWrapper#handle(java.lang.String,
     * org.eclipse.jetty.server.Request, javax.servlet.http.HttpServletRequest,
     * javax.servlet.http.HttpServletResponse)
     */
    @Override
    public void handle(String path, Request request, HttpServletRequest httpRequest, HttpServletResponse httpResponse)
            throws IOException, ServletException {
        if (!registry.isEnabled()) {
            super.handle(path, request, httpRequest, httpResponse);
            return;
        }
        final HttpChannelState state = request.getHttpChannelState();
        if (state.isInitial()) {
            state.addListener(listener);
        }
        final long startTime = registry.getNanos();
        try {
            super.handle(path, request, httpRequest, httpResponse);
        } finally {
            if (!state.isSuspended() && state.isInitial()) {
                updateResponses(httpRequest, httpResponse, startTime);
            }
            // else onCompletion will handle it.
        }
    }

    /**
     * Get a grouping identifier for metrics based on request method.
     *
     * @param method
     *            the method
     * @return the string
     */
    private String getMethodGroup(String method) {
        final HttpMethod m = HttpMethod.fromString(method);
        if (m == null) {
            return "other";
        }
        return m.asString().toLowerCase();
    }

    /**
     * Get a grouping identifier for metrics based on response status.
     *
     * @param status
     *            the status
     * @return the string
     */
    private String getStatusGroup(int status) {
        final int responseStatus = status / 100;
        if ((responseStatus < 1) || (responseStatus > 5)) {
            return "other";
        }
        return responseStatus + "xx";
    }

    /**
     * Update response based metrics such as duration.
     *
     * @param request
     *            the request
     * @param response
     *            the response
     * @param startTime
     *            the start time
     */
    private void updateResponses(HttpServletRequest request, HttpServletResponse response, long startTime) {
        final long duration = registry.getNanos() - startTime;
        registry.increment("jetty.Aggregated.Invocations", "requests");
        registry.add("jetty.Aggregated.Durations", "requests", duration);
        final String methodGroup = getMethodGroup(request.getMethod());
        registry.increment("jetty.Request.Invocations", methodGroup + "-requests");
        registry.add("jetty.Request.Durations", methodGroup + "-requests", duration);
        final String statusGroup = getStatusGroup(response.getStatus());
        registry.increment("jetty.Response.Invocations", statusGroup + "-responses");
        registry.add("jetty.Response.Durations", statusGroup + "-responses", duration);
    }
}

==> server-25.1.0-1114.113-sources.jar: com/xebialabs/deployit/server/jetty/metrics/CustomInstrumentedHandler.java <==
package com.xebialabs.deployit.server.jetty.metrics;

import com.tqdev.metrics.core.Gauge;
import com.tqdev.metrics.core.MetricRegistry;
import jakarta.servlet.AsyncEvent;
import jakarta.servlet.AsyncListener;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.eclipse.jetty.ee10.servlet.AsyncContextState;
import org.eclipse.jetty.ee10.servlet.ServletChannelState;
import org.eclipse.jetty.ee10.servlet.ServletContextRequest;
import org.eclipse.jetty.http.HttpMethod;
import org.eclipse.jetty.server.Handler;
import org.eclipse.jetty.server.Request;
import org.eclipse.jetty.server.Response;
import org.eclipse.jetty.util.Callback;
import org.eclipse.jetty.util.thread.QueuedThreadPool;

import java.io.IOException;

public class CustomInstrumentedHandler extends Handler.Wrapper {

    /** The registry to store metrics in. */
    private final MetricRegistry registry;

    /**
     * Instantiates a new instrumented handler.
     *
     * @param registry
     *            the registry
     */
    public CustomInstrumentedHandler(MetricRegistry registry) {
        this.registry = registry;
    }

    /** The listener to handle async requests consistently. */
    private AsyncListener listener = new AsyncListener() {
        private long startTime;

        @Override
        public void onTimeout(AsyncEvent event) throws IOException {
        }

        @Override
        public void onStartAsync(AsyncEvent event) throws IOException {
            startTime = registry.getNanos();
            event.getAsyncContext().addListener(this);
        }

        @Override
        public void onError(AsyncEvent event) throws IOException {
        }

        @Override
        public void onComplete(AsyncEvent event) throws IOException {
            final AsyncContextState state = (AsyncContextState) event.getAsyncContext();
            final HttpServletRequest request = (HttpServletRequest) state.getRequest();
            final HttpServletResponse response = (HttpServletResponse) state.getResponse();
            updateResponses(request, response, startTime);
        }
    };

    /*
     * (non-Javadoc)
     *
     * @see org.eclipse.jetty.server.handler.AbstractHandler#doStart()
     */
    @Override
    protected void doStart() throws Exception {
        super.doStart();

        for (int responseStatus = 1; responseStatus <= 5; responseStatus++) {
            registry.set("jetty.Response.Invocations", responseStatus + "xx-responses", 0);
            registry.set("jetty.Response.Durations", responseStatus + "xx-responses", 0);
        }
        registry.set("jetty.Response.Invocations", "other-responses", 0);
        registry.set("jetty.Response.Durations", "other-responses", 0);
        for (HttpMethod method : HttpMethod.values()) {
            String name = method.asString().toLowerCase();
            registry.set("jetty.Request.Invocations", name + "-requests", 0);
            registry.set("jetty.Request.Durations", name + "-requests", 0);
        }
        registry.set("jetty.Request.Invocations", "other-requests", 0);
        registry.set("jetty.Request.Durations", "other-requests", 0);
        registry.set("jetty.Aggregated.Invocations", "requests", 0);
        registry.set("jetty.Aggregated.Durations", "requests", 0);

        registry.set("jetty.Thread.Gauges", "threads", (Gauge) () -> getServer().getThreadPool().getThreads());
        registry.set("jetty.Thread.Gauges", "idle-threads", (Gauge) () -> getServer().getThreadPool().getIdleThreads());
        if (getServer().getThreadPool() instanceof QueuedThreadPool) {
            registry.set("jetty.Thread.Gauges", "busy-threads",
                    (Gauge) () -> ((QueuedThreadPool) getServer().getThreadPool()).getBusyThreads());
            registry.set("jetty.Thread.Gauges", "min-threads",
                    (Gauge) () -> ((QueuedThreadPool) getServer().getThreadPool()).getMinThreads());
            registry.set("jetty.Thread.Gauges", "max-threads",
                    (Gauge) () -> ((QueuedThreadPool) getServer().getThreadPool()).getMaxThreads());
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.eclipse.jetty.server.handler.HandlerWrapper#handle(java.lang.String,
     * org.eclipse.jetty.server.Request, javax.servlet.http.HttpServletRequest,
     * javax.servlet.http.HttpServletResponse)
     */
    @Override
    public boolean handle(Request request, Response response, Callback callback) throws Exception {
        if (!registry.isEnabled()) {
            return super.handle(request, response, callback);
        }
        if (request instanceof ServletContextRequest httpRequest) {
            ServletChannelState state = httpRequest.getServletRequestState();
            if (state.isInitial()) {
                state.addListener(listener);
            }
            final long startTime = registry.getNanos();
            try {
                return super.handle(request, response, callback);
            } finally {
                if (!state.isSuspended() && state.isInitial()) {
                    updateResponses(request, response, startTime);
                }
                // else onCompletion will handle it.
            }
        }
        return super.handle(request, response, callback);
    }

    /**
     * Get a grouping identifier for metrics based on request method.
     *
     * @param method
     *            the method
     * @return the string
     */
    private String getMethodGroup(String method) {
        final HttpMethod m = HttpMethod.fromString(method);
        if (m == null) {
            return "other";
        }
        return m.asString().toLowerCase();
    }

    /**
     * Get a grouping identifier for metrics based on response status.
     *
     * @param status
     *            the status
     * @return the string
     */
    private String getStatusGroup(int status) {
        final int responseStatus = status / 100;
        if ((responseStatus < 1) || (responseStatus > 5)) {
            return "other";
        }
        return responseStatus + "xx";
    }

    /**
     * Update response based metrics such as duration.
     *
     * @param request
     *            the request
     * @param response
     *            the response
     * @param startTime
     *            the start time
     */
    private void updateResponses(Request request, Response response, long startTime) {
        final long duration = registry.getNanos() - startTime;
        registry.increment("jetty.Aggregated.Invocations", "requests");
        registry.add("jetty.Aggregated.Durations", "requests", duration);
        final String methodGroup = getMethodGroup(request.getMethod());
        registry.increment("jetty.Request.Invocations", methodGroup + "-requests");
        registry.add("jetty.Request.Durations", methodGroup + "-requests", duration);
        final String statusGroup = getStatusGroup(response.getStatus());
        registry.increment("jetty.Response.Invocations", statusGroup + "-responses");
        registry.add("jetty.Response.Durations", statusGroup + "-responses", duration);
    }

    private void updateResponses(HttpServletRequest request, HttpServletResponse response, long startTime) {
        final long duration = registry.getNanos() - startTime;
        registry.increment("jetty.Aggregated.Invocations", "requests");
        registry.add("jetty.Aggregated.Durations", "requests", duration);
        final String methodGroup = getMethodGroup(request.getMethod());
        registry.increment("jetty.Request.Invocations", methodGroup + "-requests");
        registry.add("jetty.Request.Durations", methodGroup + "-requests", duration);
        final String statusGroup = getStatusGroup(response.getStatus());
        registry.increment("jetty.Response.Invocations", statusGroup + "-responses");
        registry.add("jetty.Response.Durations", statusGroup + "-responses", duration);
    }
}
